import { writable } from 'svelte/store';
import type { ReadLaterItem } from './types';

function createReadLaterStore() {
	const { subscribe, set, update } = writable<ReadLaterItem[]>([]);

	return {
		subscribe,
		add: async (url: string) => {
			const response = await fetch('/api/readlist', {
				method: 'POST',
				headers: {
					'Content-Type': 'application/json'
				},
				body: JSON.stringify({ url })
			});

			if (!response.ok) {
				throw new Error('Failed to add URL');
			}

			const item = await response.json();
			update((items) => [
				{
					...item,
					createdAt: new Date(item.createdAt),
					updatedAt: new Date(item.updatedAt),
					savedAt: new Date(item.savedAt),
					readAt: item.readAt ? new Date(item.readAt) : undefined
				},
				...items
			]);
			return item;
		},
		markRead: async (id: string) => {
			const response = await fetch(`/api/readlist/${id}/read`, {
				method: 'POST'
			});

			if (!response.ok) {
				throw new Error('Failed to mark as read');
			}

			update((items) =>
				items.map((item) =>
					item.id === id ? { ...item, readAt: new Date(), updatedAt: new Date() } : item
				)
			);
		},
		delete: async (id: string) => {
			const response = await fetch(`/api/readlist/${id}`, {
				method: 'DELETE'
			});

			if (!response.ok) {
				throw new Error('Failed to delete item');
			}

			update((items) => items.filter((item) => item.id !== id));
		},
		load: async () => {
			try {
				const response = await fetch('/api/readlist');
				if (!response.ok) {
					throw new Error('Failed to load read later items');
				}

				const items: ReadLaterItem[] = await response.json();
				if (!Array.isArray(items)) {
					set([]);
					return;
				}

				set(
					items.map((item) => ({
						...item,
						createdAt: new Date(item.createdAt),
						updatedAt: new Date(item.updatedAt),
						readAt: item.readAt ? new Date(item.readAt) : undefined
					}))
				);
			} catch (error) {
				console.error('Failed to load read later items:', error);
				set([]);
			}
		}
	};
}

export const readlist = createReadLaterStore();
